"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const config_1 = require("./config");
const metadata_1 = require("./metadata");
function Handler(config = {}) {
    return (target, methodName, descriptor) => {
        metadata_1.default.setFunctions(target.constructor, methodName, config);
        const localInvoke = transform(target, methodName, descriptor.value);
        descriptor.value = localInvoke;
        target[`__invoke_${methodName}`] = localInvoke;
        if (process.env.NODE_ENV !== 'test' && process.env.IS_LOCAL !== 'true') {
            descriptor.value = createLambdaInvokeFunction(methodName, config);
        }
        return descriptor;
    };
}
exports.Handler = Handler;
function transform(target, key, fn) {
    // eslint-disable-next-line func-names
    return function (event, context) {
        const originalArgs = [event, context];
        const transformTypes = metadata_1.default.getParams(target.constructor, key) || [];
        // spread sparse array into full array with undefineds
        const args = [...transformTypes].map((transformType) => {
            if (transformType === config_1.RequestParam.JsonBody) {
                return transformJsonBody(event);
            }
            return originalArgs.shift();
        });
        return fn.apply(this, [...args, ...originalArgs]);
    };
}
function transformJsonBody(event) {
    if (event.body && event.body !== '') {
        return JSON.parse(event.body);
    }
    return {};
}
function createLambdaInvokeFunction(methodName, config) {
    return function awsInvoke(...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const defaultName = `${process.env.SLS_DECORATORS_FUNC_PREFIX}-${methodName}`;
            const functionName = config.name || defaultName;
            console.info(`Invoking Lambda function: ${functionName}`);
            const lambda = new aws_sdk_1.Lambda();
            return new Promise((resolve, reject) => {
                lambda.invoke({
                    FunctionName: functionName,
                    InvocationType: 'Event',
                    Payload: JSON.stringify(args[0]),
                }, (err, data) => (err ? reject(err) : resolve(data)));
            });
        });
    };
}
