"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deepmerge = require("deepmerge");
const metadata_1 = require("./metadata");
function Service(config) {
    return function Decorator(constructor) {
        const functionConfigs = metadata_1.default.getFunctions(constructor);
        if (config && functionConfigs) {
            for (const [key, functionConfig] of functionConfigs) {
                metadata_1.default.setFunctions(constructor, key, deepmerge(config, functionConfig));
            }
        }
        const envProperties = metadata_1.default.getEnv(constructor);
        if (envProperties) {
            return classWithEnvProperties(constructor, envProperties);
        }
    };
}
exports.Service = Service;
function classWithEnvProperties(constructorFn, envProperties) {
    return class extends constructorFn {
        constructor(...args) {
            super(...args);
            if (Array.isArray(envProperties)) {
                for (const prop of envProperties) {
                    if (process.env[prop.envName] !== undefined) {
                        this[prop.key] = process.env[prop.envName];
                    }
                }
            }
        }
    };
}
