"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FUNCTIONS_KEY = '__SLSTS_FUNCTIONS';
const ENV_KEY = '__SLSTS_ENV_PROPERTIES';
const PARAMS_KEY = '__SLSTS_PARAMS';
class Metadata {
    static getFunction(constructor, key) {
        return safeGet(constructor, FUNCTIONS_KEY, key);
    }
    static getFunctions(constructor) {
        const functions = safeGet(constructor, FUNCTIONS_KEY);
        return functions && Object.entries(functions);
    }
    static setFunctions(constructor, key, config) {
        safeSet(constructor, [FUNCTIONS_KEY, {}], [key, config]);
    }
    static getEnv(constructor) {
        return safeGet(constructor, ENV_KEY);
    }
    static setEnv(constructor, property) {
        const length = safeGet(constructor, ENV_KEY, 'length') || 0;
        safeSet(constructor, [ENV_KEY, []], [length, property]);
    }
    static getParams(constructor, key) {
        return safeGet(constructor, PARAMS_KEY, key);
    }
    static setParams(constructor, key, index, value) {
        safeSet(constructor, [PARAMS_KEY, {}], [key, []], [index, value]);
    }
}
exports.default = Metadata;
function safeGet(target, ...keys) {
    let result = target;
    for (const key of keys) {
        if (result[key]) {
            result = result[key];
        }
        else {
            return;
        }
    }
    return result;
}
function safeSet(target, ...keys) {
    let result = target;
    for (let i = 0; i < keys.length; i++) {
        const [key, value] = keys[i];
        const lastKey = i === keys.length - 1;
        if (lastKey || !result[key]) {
            result[key] = value;
        }
        result = result[key];
    }
}
