"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const compile_1 = require("./compile");
module.exports = class ServerlessPlugin {
    constructor(serverless, options) {
        this.hooks = {
            'after:invoke:local:invoke': this.cleanup.bind(this),
            'after:invoke:invoke': this.cleanup.bind(this),
            'after:package:createDeploymentArtifacts': this.cleanup.bind(this),
            'after:deploy:function:packageFunction': this.cleanup.bind(this),
            'after:run:run': this.cleanup.bind(this),
        };
        this.serverless = serverless;
        this.options = options;
        this.servicePath = this.serverless.config.servicePath;
        this.compileFuntionsConfig();
    }
    compileFuntionsConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            this.serverless.cli.log('Adding functions to Serverless config');
            const config = yield compile_1.default(this.servicePath);
            // Object.assign(this.serverless.service.functions, config);
            const slsService = this.serverless.service;
            const serviceName = slsService['service'];
            const stageName = this.options.stage || this.options['s'] || slsService.provider.stage;
            slsService.update({
                functions: config,
                provider: {
                    environment: {
                        SLS_DECORATORS_FUNC_PREFIX: `${serviceName}-${stageName}`,
                    },
                },
            });
        });
    }
    cleanup() {
        // console.log(this.serverless.service.functions);
        fs.removeSync(path.join(this.servicePath, '/.decorators'));
    }
};
